@@ -44,7 +44,7 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
 {
     Image *images;
     Info *info;
-    volatile VALUE info_obj;
+    VALUE info_obj;
 
     if (argc > 1)
     {
@@ -74,6 +74,8 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     rm_check_image_exception(images, RetainOnError);
     rm_split(images);
 
+    RB_GC_GUARD(info_obj);
+
     return self;
 }
 
@@ -93,7 +95,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
 {
     Image *images, *new_image;
     unsigned int stack;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     // Convert the image array to an image sequence.
     images = images_from_imagelist(self);
@@ -102,11 +104,11 @@ ImageList_append(VALUE self, VALUE stack_arg)
     // otherwise left-to-right.
     stack = RTEST(stack_arg);
 
-    GetExceptionInfo(&exception);
-    new_image = AppendImages(images, stack, &exception);
+    exception = AcquireExceptionInfo();
+    new_image = AppendImages(images, stack, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_image, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_image);
 
@@ -127,16 +129,21 @@ VALUE
 ImageList_average(VALUE self)
 {
     Image *images, *new_image;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     // Convert the images array to an images sequence.
     images = images_from_imagelist(self);
 
-    GetExceptionInfo(&exception);
-    new_image = AverageImages(images, &exception);
+    exception = AcquireExceptionInfo();
+#if defined(HAVE_EVALUATEIMAGES)
+    new_image = EvaluateImages(images, MeanEvaluateOperator, exception);
+#else
+    new_image = AverageImages(images, exception);
+#endif
+
     rm_split(images);
-    rm_check_exception(&exception, new_image, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_image);
 
@@ -161,16 +168,16 @@ VALUE
 ImageList_coalesce(VALUE self)
 {
     Image *images, *new_images;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     // Convert the image array to an image sequence.
     images = images_from_imagelist(self);
 
-    GetExceptionInfo(&exception);
-    new_images = CoalesceImages(images, &exception);
+    exception = AcquireExceptionInfo();
+    new_images = CoalesceImages(images, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
@@ -197,11 +204,11 @@ ImageList_coalesce(VALUE self)
 VALUE
 ImageList_composite_layers(int argc, VALUE *argv, VALUE self)
 {
-    volatile VALUE source_images;
+    VALUE source_images;
     Image *dest, *source, *new_images;
     RectangleInfo geometry;
     CompositeOperator operator = OverCompositeOp;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     switch (argc)
     {
@@ -231,11 +238,13 @@ ImageList_composite_layers(int argc, VALUE *argv, VALUE self)
                         , new_images->page.height != 0 ? new_images->page.height : new_images->rows
                         , new_images->gravity, &geometry);
 
-    GetExceptionInfo(&exception);
-    CompositeLayers(new_images, operator, source, geometry.x, geometry.y, &exception);
+    exception = AcquireExceptionInfo();
+    CompositeLayers(new_images, operator, source, geometry.x, geometry.y, exception);
     rm_split(source);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
+
+    RB_GC_GUARD(source_images);
 
     return rm_imagelist_from_images(new_images);
 }
@@ -255,14 +264,14 @@ VALUE
 ImageList_deconstruct(VALUE self)
 {
     Image *new_images, *images;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     images = images_from_imagelist(self);
-    GetExceptionInfo(&exception);
-    new_images = DeconstructImages(images, &exception);
+    exception = AcquireExceptionInfo();
+    new_images = DeconstructImages(images, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
@@ -284,7 +293,7 @@ ImageList_display(VALUE self)
 {
     Image *images;
     Info *info;
-    volatile VALUE info_obj;
+    VALUE info_obj;
 
     // Create a new Info object to use with this call
     info_obj = rm_info_new();
@@ -297,6 +306,8 @@ ImageList_display(VALUE self)
     rm_split(images);
     rm_check_image_exception(images, RetainOnError);
 
+    RB_GC_GUARD(info_obj);
+
     return self;
 }
 
@@ -317,20 +328,20 @@ VALUE
 ImageList_flatten_images(VALUE self)
 {
     Image *images, *new_image;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     images = images_from_imagelist(self);
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
 #if defined(HAVE_ENUM_FLATTENLAYER)
-    new_image = MergeImageLayers(images, FlattenLayer, &exception);
+    new_image = MergeImageLayers(images, FlattenLayer, exception);
 #else
-    new_image = FlattenImages(images, &exception);
+    new_image = FlattenImages(images, exception);
 #endif
 
     rm_split(images);
-    rm_check_exception(&exception, new_image, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_image);
 
@@ -360,7 +371,7 @@ ImageList_fx(int argc, VALUE *argv, VALUE self)
     Image *images, *new_image;
     char *expression;
     ChannelType channels;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
 
     channels = extract_channels(&argc, argv);
@@ -378,11 +389,11 @@ ImageList_fx(int argc, VALUE *argv, VALUE self)
     expression = StringValuePtr(argv[0]);
 
     images = images_from_imagelist(self);
-    GetExceptionInfo(&exception);
-    new_image = FxImageChannel(images, channels, expression, &exception);
+    exception = AcquireExceptionInfo();
+    new_image = FxImageChannel(images, channels, expression, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_image, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_image);
 
@@ -412,10 +423,11 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
     Image *images, *new_images = NULL;
     Image *map;
     unsigned int dither = MagickFalse;
-    volatile VALUE scene, new_imagelist, t;
-    ExceptionInfo exception;
+    VALUE scene, new_imagelist, t;
+    ExceptionInfo *exception;
 
 #if defined(HAVE_REMAPIMAGES)
+    QuantizeInfo quantize_info;
     rb_warning("ImageList#map is deprecated. Use ImageList#remap instead.");
 #endif
 
@@ -434,18 +446,24 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
 
 
     // Convert image array to image sequence, clone image sequence.
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
     images = images_from_imagelist(self);
-    new_images = CloneImageList(images, &exception);
+    new_images = CloneImageList(images, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
     // Call ImageMagick
+#if defined(HAVE_REMAPIMAGES)
+    GetQuantizeInfo(&quantize_info);
+    quantize_info.dither = dither;
+    (void) RemapImages(&quantize_info, new_images, map);
+#else
     (void) MapImages(new_images, map, dither);
+#endif
     rm_check_image_exception(new_images, DestroyOnError);
 
     // Set @scene in new ImageList object to same value as in self.
@@ -453,6 +471,10 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
     scene = rb_iv_get(self, "@scene");
     (void) imagelist_scene_eq(new_imagelist, scene);
 
+    RB_GC_GUARD(scene);
+    RB_GC_GUARD(new_imagelist);
+    RB_GC_GUARD(t);
+
     return new_imagelist;
 }
 
@@ -473,10 +495,10 @@ ImageList_map(int argc, VALUE *argv, VALUE self)
 VALUE
 ImageList_montage(VALUE self)
 {
-    volatile VALUE montage_obj;
+    VALUE montage_obj;
     Montage *montage;
     Image *new_images, *images;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     // Create a new instance of the Magick::Montage class
     montage_obj = rm_montage_new();
@@ -501,16 +523,18 @@ ImageList_montage(VALUE self)
         }
     }
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
     // MontageImage can return more than one image.
-    new_images = MontageImages(images, montage->info, &exception);
+    new_images = MontageImages(images, montage->info, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
+    RB_GC_GUARD(montage_obj);
+
     return rm_imagelist_from_images(new_images);
 }
 
@@ -533,7 +557,7 @@ VALUE
 ImageList_morph(VALUE self, VALUE nimages)
 {
     Image *images, *new_images;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
     long number_images;
 
 
@@ -544,12 +568,12 @@ ImageList_morph(VALUE self, VALUE nimages)
         rb_raise(rb_eArgError, "number of intervening images must be > 0");
     }
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
     images = images_from_imagelist(self);
-    new_images = MorphImages(images, (unsigned long)number_images, &exception);
+    new_images = MorphImages(images, (unsigned long)number_images, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
@@ -570,20 +594,20 @@ VALUE
 ImageList_mosaic(VALUE self)
 {
     Image *images, *new_image;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
     images = images_from_imagelist(self);
 
 #if defined(HAVE_ENUM_MOSAICLAYER)
-    new_image = MergeImageLayers(images, MosaicLayer, &exception);
+    new_image = MergeImageLayers(images, MosaicLayer, exception);
 #else
-    new_image = MosaicImages(images, &exception);
+    new_image = MosaicImages(images, exception);
 #endif
 
     rm_split(images);
-    rm_check_exception(&exception, new_image, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_image);
 
@@ -606,12 +630,12 @@ ImageList_optimize_layers(VALUE self, VALUE method)
 {
     Image *images, *new_images, *new_images2;
     LAYERMETHODTYPE mthd;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
     QuantizeInfo quantize_info;
 
     new_images2 = NULL;     // defeat "unused variable" message
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 #if defined(HAVE_TYPE_IMAGELAYERMETHOD)
     VALUE_TO_ENUM(method, mthd, ImageLayerMethod);
 #else
@@ -622,22 +646,22 @@ ImageList_optimize_layers(VALUE self, VALUE method)
     switch (mthd)
     {
         case CoalesceLayer:
-            new_images = CoalesceImages(images, &exception);
+            new_images = CoalesceImages(images, exception);
             break;
         case DisposeLayer:
-            new_images = DisposeImages(images, &exception);
+            new_images = DisposeImages(images, exception);
             break;
         case OptimizeTransLayer:
             new_images = clone_imagelist(images);
-            OptimizeImageTransparency(new_images, &exception);
+            OptimizeImageTransparency(new_images, exception);
             break;
         case RemoveDupsLayer:
             new_images = clone_imagelist(images);
-            RemoveDuplicateLayers(&new_images, &exception);
+            RemoveDuplicateLayers(&new_images, exception);
             break;
         case RemoveZeroLayer:
             new_images = clone_imagelist(images);
-            RemoveZeroDelayLayers(&new_images, &exception);
+            RemoveZeroDelayLayers(&new_images, exception);
             break;
         case CompositeLayer:
             rm_split(images);
@@ -645,19 +669,19 @@ ImageList_optimize_layers(VALUE self, VALUE method)
             break;
             // In 6.3.4-ish, OptimizeImageLayer replaced OptimizeLayer
         case OptimizeImageLayer:
-            new_images = OptimizeImageLayers(images, &exception);
+            new_images = OptimizeImageLayers(images, exception);
             break;
             // and OptimizeLayer became a "General Purpose, GIF Animation Optimizer" (ref. mogrify.c)
         case OptimizeLayer:
-            new_images = CoalesceImages(images, &exception);
+            new_images = CoalesceImages(images, exception);
             rm_split(images);
-            rm_check_exception(&exception, new_images, DestroyOnError);
-            new_images2 = OptimizeImageLayers(new_images, &exception);
+            rm_check_exception(exception, new_images, DestroyOnError);
+            new_images2 = OptimizeImageLayers(new_images, exception);
             DestroyImageList(new_images);
-            rm_check_exception(&exception, new_images2, DestroyOnError);
+            rm_check_exception(exception, new_images2, DestroyOnError);
             new_images = new_images2;
-            OptimizeImageTransparency(new_images, &exception);
-            rm_check_exception(&exception, new_images, DestroyOnError);
+            OptimizeImageTransparency(new_images, exception);
+            rm_check_exception(exception, new_images, DestroyOnError);
             // mogrify supports -dither here. We don't.
 #if defined(HAVE_REMAPIMAGE)
             GetQuantizeInfo(&quantize_info);
@@ -667,31 +691,31 @@ ImageList_optimize_layers(VALUE self, VALUE method)
 #endif
             break;
         case OptimizePlusLayer:
-            new_images = OptimizePlusImageLayers(images, &exception);
+            new_images = OptimizePlusImageLayers(images, exception);
             break;
         case CompareAnyLayer:
         case CompareClearLayer:
         case CompareOverlayLayer:
-            new_images = CompareImageLayers(images, mthd, &exception);
+            new_images = CompareImageLayers(images, mthd, exception);
             break;
 #if defined(HAVE_ENUM_MOSAICLAYER)
         case MosaicLayer:
-            new_images = MergeImageLayers(images, mthd, &exception);
+            new_images = MergeImageLayers(images, mthd, exception);
             break;
 #endif
 #if defined(HAVE_ENUM_FLATTENLAYER)
         case FlattenLayer:
-            new_images = MergeImageLayers(images, mthd, &exception);
+            new_images = MergeImageLayers(images, mthd, exception);
             break;
 #endif
 #if defined(HAVE_ENUM_MERGELAYER)
         case MergeLayer:
-            new_images = MergeImageLayers(images, mthd, &exception);
+            new_images = MergeImageLayers(images, mthd, exception);
             break;
 #endif
 #if defined(HAVE_ENUM_TRIMBOUNDSLAYER)
         case TrimBoundsLayer:
-            new_images = MergeImageLayers(images, mthd, &exception);
+            new_images = MergeImageLayers(images, mthd, exception);
             break;
 #endif
         default:
@@ -701,8 +725,8 @@ ImageList_optimize_layers(VALUE self, VALUE method)
     }
 
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     rm_ensure_result(new_images);
 
@@ -742,7 +766,7 @@ ImageList_new(void)
 VALUE
 rm_imagelist_from_images(Image *images)
 {
-    volatile VALUE new_imagelist;
+    VALUE new_imagelist;
     Image *image;
 
     if (!images)
@@ -759,6 +783,9 @@ rm_imagelist_from_images(Image *images)
     }
 
     (void) rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
+
+    RB_GC_GUARD(new_imagelist);
+
     return new_imagelist;
 }
 
@@ -778,7 +805,7 @@ images_from_imagelist(VALUE imagelist)
 {
     long x, len;
     Image *head = NULL;
-    volatile VALUE images, t;
+    VALUE images, t;
 
     len = check_imagelist_length(imagelist);
 
@@ -789,9 +816,17 @@ images_from_imagelist(VALUE imagelist)
 
         t = rb_ary_entry(images, x);
         image = rm_check_destroyed(t);
+        // avoid a loop in this linked imagelist, issue #202
+        if (head == image || GetPreviousImageInList(image) != NULL)
+        {
+            image = rm_clone_image(image);
+        }
         AppendImageToList(&head, image);
     }
 
+    RB_GC_GUARD(images);
+    RB_GC_GUARD(t);
+
     return head;
 }
 
@@ -825,7 +860,10 @@ imagelist_scene_eq(VALUE imagelist, VALUE scene)
 static long
 imagelist_length(VALUE imagelist)
 {
-    volatile VALUE images = rb_iv_get(imagelist, "@images");
+    VALUE images = rb_iv_get(imagelist, "@images");
+
+    RB_GC_GUARD(images);
+
     return RARRAY_LEN(images);
 }
 
@@ -881,20 +919,20 @@ static Image *
 clone_imagelist(Image *images)
 {
     Image *new_imagelist = NULL, *image, *clone;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
     image = GetFirstImageInList(images);
     while (image)
     {
-        clone = CloneImage(image, 0, 0, MagickTrue, &exception);
-        rm_check_exception(&exception, new_imagelist, DestroyOnError);
+        clone = CloneImage(image, 0, 0, MagickTrue, exception);
+        rm_check_exception(exception, new_imagelist, DestroyOnError);
         AppendImageToList(&new_imagelist, clone);
         image = GetNextImageInList(image);
     }
 
-    (void) DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(exception);
     return new_imagelist;
 }
 
@@ -929,8 +967,8 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     Image *images, *new_images;
     Image *new_image;
     QuantizeInfo quantize_info;
-    ExceptionInfo exception;
-    volatile VALUE new_imagelist, scene;
+    ExceptionInfo *exception;
+    VALUE new_imagelist, scene;
 
     GetQuantizeInfo(&quantize_info);
 
@@ -963,18 +1001,18 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
 
 
     // Convert image array to image sequence, clone image sequence.
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
     images = images_from_imagelist(self);
-    new_images = CloneImageList(images, &exception);
+    new_images = CloneImageList(images, exception);
     rm_split(images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, new_images, DestroyOnError);
 
     rm_ensure_result(new_images);
 
 
     (void) QuantizeImages(&quantize_info, new_images);
-    rm_check_exception(&exception, new_images, DestroyOnError);
+    rm_check_exception(exception, new_images, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     // Create new ImageList object, convert mapped image sequence to images,
     // append to images array.
@@ -988,6 +1026,9 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     scene = rb_iv_get(self, "@scene");
     (void) rb_iv_set(new_imagelist, "@scene", scene);
 
+    RB_GC_GUARD(new_imagelist);
+    RB_GC_GUARD(scene);
+
     return new_imagelist;
 }
 
@@ -1020,8 +1061,9 @@ ImageList_remap(int argc, VALUE *argv, VALUE self)
 
     if (argc > 0 && argv[0] != Qnil)
     {
-        volatile VALUE t = rm_cur_image(argv[0]);
+        VALUE t = rm_cur_image(argv[0]);
         remap_image = rm_check_destroyed(t);
+        RB_GC_GUARD(t);
     }
 
     GetQuantizeInfo(&quantize_info);
@@ -1075,11 +1117,11 @@ ImageList_to_blob(VALUE self)
 {
     Image *images, *image;
     Info *info;
-    volatile VALUE info_obj;
-    volatile VALUE blob_str;
+    VALUE info_obj;
+    VALUE blob_str;
     void *blob = NULL;
     size_t length = 0;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
@@ -1087,9 +1129,9 @@ ImageList_to_blob(VALUE self)
     // Convert the images array to an images sequence.
     images = images_from_imagelist(self);
 
-    GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, MagickTrue, &exception);
-    rm_check_exception(&exception, images, RetainOnError);
+    exception = AcquireExceptionInfo();
+    (void) SetImageInfo(info, MagickTrue, exception);
+    rm_check_exception(exception, images, RetainOnError);
 
     if (*info->magick != '\0')
     {
@@ -1109,8 +1151,8 @@ ImageList_to_blob(VALUE self)
     // can happen is that there's only one image or the format
     // doesn't support multi-image files.
     info->adjoin = MagickTrue;
-    blob = ImagesToBlob(info, images, &length, &exception);
-    if (blob && exception.severity >= ErrorException)
+    blob = ImagesToBlob(info, images, &length, exception);
+    if (blob && exception->severity >= ErrorException)
     {
         magick_free((void*)blob);
         blob = NULL;
@@ -1118,7 +1160,7 @@ ImageList_to_blob(VALUE self)
     }
     rm_split(images);
     CHECK_EXCEPTION()
-    (void) DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(exception);
 
 
     if (length == 0 || !blob)
@@ -1129,6 +1171,9 @@ ImageList_to_blob(VALUE self)
     blob_str = rb_str_new(blob, (long)length);
     magick_free((void*)blob);
 
+    RB_GC_GUARD(info_obj);
+    RB_GC_GUARD(blob_str);
+
     return blob_str;
 }
 
@@ -1152,9 +1197,9 @@ ImageList_write(VALUE self, VALUE file)
     Image *images, *img;
     Info *info;
     const MagickInfo *m;
-    volatile VALUE info_obj;
+    VALUE info_obj;
     unsigned long scene;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
@@ -1166,7 +1211,12 @@ ImageList_write(VALUE self, VALUE file)
 
         // Ensure file is open - raise error if not
         GetOpenFile(file, fptr);
+#if defined(_WIN32)
+        add_format_prefix(info, fptr->pathv);
+        SetImageInfoFile(info, NULL);
+#else
         SetImageInfoFile(info, GetReadFile(fptr));
+#endif
     }
     else
     {
@@ -1186,13 +1236,13 @@ ImageList_write(VALUE self, VALUE file)
     }
 
     // Find out if the format supports multi-images files.
-    GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, MagickTrue, &exception);
-    rm_check_exception(&exception, images, RetainOnError);
+    exception = AcquireExceptionInfo();
+    (void) SetImageInfo(info, MagickTrue, exception);
+    rm_check_exception(exception, images, RetainOnError);
 
-    m = GetMagickInfo(info->magick, &exception);
-    rm_check_exception(&exception, images, RetainOnError);
-    (void) DestroyExceptionInfo(&exception);
+    m = GetMagickInfo(info->magick, exception);
+    rm_check_exception(exception, images, RetainOnError);
+    (void) DestroyExceptionInfo(exception);
 
     // Tell WriteImage if we want a multi-images file.
     if (imagelist_length(self) > 1L && m->adjoin)
@@ -1213,5 +1263,8 @@ ImageList_write(VALUE self, VALUE file)
     }
 
     rm_split(images);
+
+    RB_GC_GUARD(info_obj);
+
     return self;
 }
