@@ -585,13 +585,15 @@ rm_fuzz_to_dbl(VALUE fuzz_arg)
 Quantum
 rm_app2quantum(VALUE obj)
 {
-    volatile VALUE v = obj;
+    VALUE v = obj;
 
     if (TYPE(obj) == T_FLOAT)
     {
         v = rb_funcall(obj, rm_ID_to_i, 0);
     }
 
+    RB_GC_GUARD(v);
+
     return NUM2QUANTUM(v);
 }
 
@@ -627,13 +629,13 @@ VALUE
 rm_pixelpacket_to_color_name(Image *image, PixelPacket *color)
 {
     char name[MaxTextExtent];
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
-    (void) QueryColorname(image, color, X11Compliance, name, &exception);
+    (void) QueryColorname(image, color, X11Compliance, name, exception);
     CHECK_EXCEPTION()
-    (void) DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(exception);
 
     return rb_str_new2(name);
 }
@@ -661,7 +663,7 @@ rm_pixelpacket_to_color_name_info(Info *info, PixelPacket *color)
 {
     Image *image;
     Info *my_info;
-    volatile VALUE color_name;
+    VALUE color_name;
 
     my_info = info ? info : CloneImageInfo(NULL);
 
@@ -674,6 +676,8 @@ rm_pixelpacket_to_color_name_info(Info *info, PixelPacket *color)
         (void) DestroyImageInfo(my_info);
     }
 
+    RB_GC_GUARD(color_name);
+
     return color_name;
 }
 
@@ -698,11 +702,11 @@ rm_write_temp_image(Image *image, char *temp_name)
 #define TMPNAM_CLASS_VAR "@@_tmpnam_"
 
     MagickBooleanType okay;
-    ExceptionInfo exception;
-    volatile VALUE id_value;
+    ExceptionInfo *exception;
+    VALUE id_value;
     int id;
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
 
     // 'id' is always the value of its previous use
@@ -722,14 +726,15 @@ rm_write_temp_image(Image *image, char *temp_name)
     sprintf(temp_name, "mpri:%d", id);
 
     // Omit "mpri:" from filename to form the key
-    okay = SetImageRegistry(ImageRegistryType, temp_name+5, image, &exception);
+    okay = SetImageRegistry(ImageRegistryType, temp_name+5, image, exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    DestroyExceptionInfo(exception);
     if (!okay)
     {
         rb_raise(rb_eRuntimeError, "SetImageRegistry failed.");
     }
 
+    RB_GC_GUARD(id_value);
 }
 
 
@@ -789,13 +794,17 @@ rm_not_implemented(void)
 void
 rm_magick_error(const char *msg, const char *loc)
 {
-    volatile VALUE exc, mesg, extra;
+    VALUE exc, mesg, extra;
 
     mesg = rb_str_new2(msg);
     extra = loc ? rb_str_new2(loc) : Qnil;
 
     exc = rb_funcall(Class_ImageMagickError, rm_ID_new, 2, mesg, extra);
     (void) rb_funcall(rb_cObject, rb_intern("raise"), 1, exc);
+
+    RB_GC_GUARD(exc);
+    RB_GC_GUARD(mesg);
+    RB_GC_GUARD(extra);
 }
 
 
@@ -820,7 +829,7 @@ ImageMagickError_initialize(int argc, VALUE *argv, VALUE self)
 {
     VALUE super_argv[1] = {(VALUE)0};
     int super_argc = 0;
-    volatile VALUE extra = Qnil;
+    VALUE extra = Qnil;
 
     switch(argc)
     {
@@ -838,6 +847,7 @@ ImageMagickError_initialize(int argc, VALUE *argv, VALUE self)
     (void) rb_call_super(super_argc, (const VALUE *)super_argv);
     (void) rb_iv_set(self, "@"MAGICK_LOC, extra);
 
+    RB_GC_GUARD(extra);
 
     return self;
 }
@@ -922,21 +932,24 @@ void rm_set_user_artifact(Image *images, Info *info)
 void
 rm_get_optional_arguments(VALUE img)
 {
-  volatile VALUE OptionalMethodArguments;
-  volatile VALUE opt_args;
-  VALUE argv[1];
+    VALUE optional_method_arguments;
+    VALUE opt_args;
+    VALUE argv[1];
 
-  // opt_args = Magick::OptionalMethodArguments.new(img)
-  // opt_args.instance_eval { block }
-  if (rb_block_given_p())
-  {
-      OptionalMethodArguments = rb_const_get_from(Module_Magick, rb_intern("OptionalMethodArguments"));
-      argv[0] = img;
-      opt_args = rb_class_new_instance(1, argv, OptionalMethodArguments);
-      (void) rb_obj_instance_eval(0, NULL, opt_args);
-  }
+    // opt_args = Magick::OptionalMethodArguments.new(img)
+    // opt_args.instance_eval { block }
+    if (rb_block_given_p())
+    {
+        optional_method_arguments = rb_const_get_from(Module_Magick, rb_intern("OptionalMethodArguments"));
+        argv[0] = img;
+        opt_args = rb_class_new_instance(1, argv, optional_method_arguments);
+        (void) rb_obj_instance_eval(0, NULL, opt_args);
+    }
 
-  return;
+    RB_GC_GUARD(optional_method_arguments);
+    RB_GC_GUARD(opt_args);
+
+    return;
 }
 
 
@@ -1159,7 +1172,7 @@ rm_exif_by_entry(Image *image)
     const char *property, *value;
     char *str;
     size_t len = 0, property_l, value_l;
-    volatile VALUE v;
+    VALUE v;
 
     (void) GetImageProperty(image, "exif:*");
     ResetImagePropertyIterator(image);
@@ -1223,6 +1236,9 @@ rm_exif_by_entry(Image *image)
 
     v = rb_str_new(str, len);
     xfree(str);
+
+    RB_GC_GUARD(v);
+
     return v;
 }
 
@@ -1245,7 +1261,7 @@ rm_exif_by_number(Image *image)
     const char *property, *value;
     char *str;
     size_t len = 0, property_l, value_l;
-    volatile VALUE v;
+    VALUE v;
 
     (void) GetImageProperty(image, "exif:!");
     ResetImagePropertyIterator(image);
@@ -1309,6 +1325,9 @@ rm_exif_by_number(Image *image)
 
     v = rb_str_new(str, len);
     xfree(str);
+
+    RB_GC_GUARD(v);
+
     return v;
 }
 
@@ -1389,16 +1408,16 @@ Image *
 rm_clone_image(Image *image)
 {
     Image *clone;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
-    GetExceptionInfo(&exception);
-    clone = CloneImage(image, 0, 0, MagickTrue, &exception);
+    exception = AcquireExceptionInfo();
+    clone = CloneImage(image, 0, 0, MagickTrue, exception);
     if (!clone)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
     }
-    rm_check_exception(&exception, clone, DestroyOnError);
-    (void) DestroyExceptionInfo(&exception);
+    rm_check_exception(exception, clone, DestroyOnError);
+    (void) DestroyExceptionInfo(exception);
 
     return clone;
 }
@@ -1426,8 +1445,8 @@ rm_progress_monitor(
     const MagickSizeType sp,
     void *client_data)
 {
-    volatile VALUE rval;
-    volatile VALUE method, offset, span;
+    VALUE rval;
+    VALUE method, offset, span;
 
     tag = tag;      // defeat gcc message
 
@@ -1443,6 +1462,11 @@ rm_progress_monitor(
 
     rval = rb_funcall((VALUE)client_data, rm_ID_call, 3, method, offset, span);
 
+    RB_GC_GUARD(rval);
+    RB_GC_GUARD(method);
+    RB_GC_GUARD(offset);
+    RB_GC_GUARD(span);
+
     return RTEST(rval) ? MagickTrue : MagickFalse;
 }
 
@@ -1486,7 +1510,7 @@ rm_split(Image *image)
 void
 rm_check_image_exception(Image *imglist, ErrorRetention retention)
 {
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
     Image *badboy = NULL;
     Image *image;
 
@@ -1495,7 +1519,7 @@ rm_check_image_exception(Image *imglist, ErrorRetention retention)
         return;
     }
 
-    GetExceptionInfo(&exception);
+    exception = AcquireExceptionInfo();
 
     // Find the image with the highest severity
     image = GetFirstImageInList(imglist);
@@ -1506,7 +1530,7 @@ rm_check_image_exception(Image *imglist, ErrorRetention retention)
             if (!badboy || image->exception.severity > badboy->exception.severity)
             {
                 badboy = image;
-                InheritException(&exception, &badboy->exception);
+                InheritException(exception, &badboy->exception);
             }
 
             ClearMagickException(&image->exception);
@@ -1516,10 +1540,10 @@ rm_check_image_exception(Image *imglist, ErrorRetention retention)
 
     if (badboy)
     {
-        rm_check_exception(&exception, imglist, retention);
+        rm_check_exception(exception, imglist, retention);
     }
 
-    (void) DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(exception);
 }
 
 
@@ -1609,7 +1633,7 @@ rm_error_handler(const ExceptionType severity, const char *reason, const char *d
 void
 rm_fatal_error_handler(const ExceptionType severity, const char *reason, const char *description)
 {
-    rb_raise(Class_FatalImageMagickError, GetLocaleExceptionMessage(severity, reason));
+    rb_raise(Class_FatalImageMagickError, "%s", GetLocaleExceptionMessage(severity, reason));
     description = description;
 }
 
@@ -1649,7 +1673,7 @@ handle_exception(ExceptionInfo *exception, Image *imglist, ErrorRetention retent
             exception->description ? ": " : "",
             exception->description ? GetLocaleExceptionMessage(exception->severity, exception->description) : "");
         msg[sizeof(msg)-1] = '\0';
-        rb_warning(msg);
+        rb_warning("%s", msg);
 
         // Caller deletes ExceptionInfo...
 
